from rest_framework.views import APIView
from rest_framework.response import Response
from slackchat.serializers import ChannelCMSSerializer, ChannelSerializer
from slackchat.models import Channel, ChatType, User
from slackchat.celery import post_webhook_republish, post_webhook_unpublish
from slackchat.authentication import TokenAPIAuthentication


channelNotFound404 = Response(
    {"detail": "Channel not found. It may have been deleted."}, 404
)

typeNotFound404 = Response(
    {"detail": "Chat type not found. It may have been deleted."}, 404
)

userNotFound404 = Response(
    {"detail": "User not found. It may have been deleted."}, 404
)

notUnique400 = Response(
    {
        "detail": "This publish path is already being used. Change the path"
        + " or chat type and try again."
    },
    400,
)


class ChannelDeserializer(APIView):
    """
    View to handle data from CMS.
    """

    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = ()

    @staticmethod
    def is_path_unique(pk, channel_path, type_path):
        chans = Channel.objects.filter(
            publish_path=channel_path
        ).select_related("chat_type")

        for chan in chans:
            if (
                str(pk) != str(chan.pk)
                and chan.chat_type.publish_path == type_path
            ):
                return False

        return True

    def handle_webhook(self, c):
        if c.published:
            post_webhook_republish.delay(
                ChannelSerializer(c).data, c.chat_type.name
            )
        else:
            post_webhook_unpublish.delay(
                ChannelSerializer(c).data, c.chat_type.name
            )

    def get(self, request, format=None):
        return Response(200)

    def post(self, request, format=None):
        """
        Add a new Channel.
        """
        data = request.data.copy()

        # Get chat type record
        try:
            ct = ChatType.objects.get(pk=data.pop("chat_type"))
            data["chat_type"] = ct
        except ChatType.DoesNotExist:
            return typeNotFound404

        if not self.is_path_unique(
            None, data["publish_path"], ct.publish_path
        ):
            return notUnique400

        # Get user record
        try:
            u = User.objects.get(pk=data.pop("owner"))
            data["owner"] = u
        except User.DoesNotExist:
            return userNotFound404

        c = Channel(**data)
        c.save()

        self.handle_webhook(c)

        return Response(
            {
                "text": "Channel saved.",
                "method": "POST",
                "saved": ChannelCMSSerializer(c).data,
            },
            200,
        )

    def patch(self, request, format=None):
        """
        Update an existing Channel
        """
        data = request.data.copy()

        # Get chat type record
        try:
            ct = ChatType.objects.get(id=data.pop("chat_type"))
            data["chat_type"] = ct
        except ChatType.DoesNotExist:
            return typeNotFound404

        if not self.is_path_unique(
            data["id"], data["publish_path"], ct.publish_path
        ):
            return notUnique400

        # Get channel record
        try:
            c = Channel.objects.get(id=data.pop("id"))
        except Channel.DoesNotExist:
            return channelNotFound404

        # Save new data
        for key, value in data.items():
            setattr(c, key, value)
        c.save()

        self.handle_webhook(c)

        return Response(
            {
                "text": "Channel saved.",
                "method": "PATCH",
                "saved": ChannelCMSSerializer(c).data,
            },
            200,
        )



from django.urls import include, path
from rest_framework import routers

from .views import Events, CMSDetail, CMSList, ChannelDeserializer
from .viewsets import ChannelViewset, ChatTypeViewset

router = routers.DefaultRouter()
router.register(r"channels", ChannelViewset, base_name="slackchat-channel")
router.register(
    r"chat-types", ChatTypeViewset, base_name="slackchat-chat-types"
)

urlpatterns = [
    path("api/cms/", ChannelDeserializer.as_view(), name="cms-api"),
    path("api/", include(router.urls)),
    path("events/", Events.as_view()),
    path("cms/", CMSList.as_view(), name="cms-list"),
    path("cms/new/", CMSDetail.as_view(), name="cms-detail-new"),
    path("cms/<slug:id>/edit/", CMSDetail.as_view(), name="cms-detail-edit"),
]



# flake8: noqa
from .attachment import AttachmentSerializer
from .channel import (
    ChannelSerializer,
    ChannelListSerializer,
    ChannelCMSSerializer,
)
from .chat_type import ChatTypeSerializer
from .message import MessageSerializer
from .reaction import ReactionSerializer
from .user import UserSerializer, UserCMSSerializer
from .webhook import WebhookSerializer



import uuid

from django.forms import widgets
from django.utils.safestring import mark_safe


class MarkdownEditorWidget(widgets.Textarea):
    def render(self, name, value, attrs=None, renderer=None):
        if "class" not in attrs.keys():
            attrs["class"] = ""

        attrs["class"] += " markdown-editor"

        attrs["data-uuid"] = str(uuid.uuid4())

        html = super(MarkdownEditorWidget, self).render(
            name, value, attrs, renderer
        )

        return mark_safe(html)

    class Media:
        js = (
            "https://cdnjs.cloudflare.com/ajax/libs/simplemde/1.11.2/simplemde.min.js",  # noqa: E501
            "slackchat/js/editor.js",
        )

        css = {
            "all": (
                "https://cdnjs.cloudflare.com/ajax/libs/simplemde/1.11.2/simplemde.min.css",  # noqa: E501
                "slackchat/css/editor.css",
            )
        }



from slackchat.models import Channel, ChatType, User
from slackchat.serializers import (
    ChannelCMSSerializer,
    ChatTypeSerializer,
    UserCMSSerializer,
)
from slackchat.conf import settings
from django.shortcuts import get_object_or_404
from .base import CMSBase


class CMSDetail(CMSBase):
    template_name = "detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context["all_chat_types"] = CMSBase.prep_data_for_injection(
            ChatType.objects.all(), ChatTypeSerializer, many=True
        )

        if "id" in self.kwargs:
            context["active_page"] = "edit"

            context["data"] = CMSBase.prep_data_for_injection(
                get_object_or_404(Channel, pk=self.kwargs["id"]),
                ChannelCMSSerializer,
            )

            context["user"] = CMSBase.prep_data_for_injection({}, None)
        else:
            context["active_page"] = "new"

            try:
                user = User.objects.get(email=self.request.user)
            except User.DoesNotExist:
                user = get_object_or_404(User, api_id=settings.DEFAULT_OWNER)

            context["user"] = CMSBase.prep_data_for_injection(
                user,
                UserCMSSerializer,
            )
            context["data"] = CMSBase.prep_data_for_injection({}, None)
            context["chat_type"] = CMSBase.prep_data_for_injection({}, None)

        return context
